@@ -89,10 +89,11 @@ missing_val = set([Missing])
 
 
 class Detector(object):
-    def __init__(self, parent_scanner=None, dim_eq=None):
+    def __init__(self, parent_scanner=None, dim_eq=None, allow_need_convert=False):
         self._scanners = None
         self.objectclss = set()
         self.count_elem = 0
+        self.allow_need_convert = allow_need_convert
 
         if not dim_eq:
             dim_eq = DimEqualizer()
@@ -155,7 +156,7 @@ class Detector(object):
         return restypes[0]
 
     def _findAcceptableDescendants(self, seq, scanner=None):
-        schildren = [sc(self) for sc in _scanchildren[scanner.__class__]]
+        schildren = [sc(self) for sc in _scanchildren[scanner.__class__] if (sc.need_convert == True and self.allow_need_convert == True) or sc.need_convert == False]
        
         res = set()
         for sc in schildren:
@@ -392,6 +393,7 @@ class DimEqualizer(object):
 
 
 class TypeScanner(object):
+    need_convert=False
     typecls = rtypes.TypeAny
     good_cls = set()
 
@@ -426,7 +428,7 @@ class TypeScanner(object):
     def getSubDetector(self, id=0):
         id = 'subdetector_' + str(id)
         if id not in self.__dict__:
-            d = Detector(self, self.detector.dim_eq)
+            d = Detector(self, self.detector.dim_eq, self.detector.allow_need_convert)
             setattr(self, id, d)
             if self.detector.count_elem:
                 d.processSeq([Missing] * self.detector.count_elem)
@@ -769,6 +771,7 @@ class StringScanner(TypeScanner):
 registerTypeScanner(StringScanner)
 
 class StringRealScanner(StringScanner):
+    need_convert=True
     parentcls=StringScanner
     missing_str = set(["", "NA", "N/A","NaN", "nan", "--", "?", "null"])
     def __init__(self, detector):
@@ -776,53 +779,53 @@ class StringRealScanner(StringScanner):
         self.has_missing = False
     
     def unregister(self, create_parent=False):
-        res = super(StringRealScanner,self).unregister(self, create_parent)
+        res = super(StringRealScanner,self).unregister(create_parent)
         if create_parent:
             res.max_nchars = self.max_nchars
         return res
 
     def getType(self):
-        return rtypes.TypeReal64(self.detector.hasMissing() or self.has_missing, needscast=True)
+        return rtypes.TypeReal64(self.detector.hasMissing() or self.has_missing)
 
     def scan(self, seq):
         res = StringScanner.scan(self, seq)
         if res:
-            for elem in seq:
+            for elem in seq.ravel():
                 try:
                     float(elem)
-                except ValueError:
+                except (ValueError, TypeError):
                     if elem.__class__ in missing_cls or elem in self.missing_str:
                         self.has_missing = True
                     else:
                         return False
         return res
-#registerTypeScanner(StringRealScanner)
+registerTypeScanner(StringRealScanner)
 
 class StringIntScanner(StringRealScanner):
     parentcls=StringRealScanner
     
     def unregister(self, create_parent=False):
-        res = super(StringIntScanner,self).unregister(self, create_parent)
+        res = super(StringIntScanner,self).unregister(create_parent)
         if create_parent:
             res.has_missing = self.has_missing
         return res
 
     def getType(self):
-        return rtypes.TypeInt64(self.detector.hasMissing() or self.has_missing, needscast=True)
+        return rtypes.TypeInt64(self.detector.hasMissing() or self.has_missing)
 
     def scan(self, seq):
         res = StringScanner.scan(self, seq)
         if res:
-            for elem in seq:
+            for elem in seq.ravel():
                 try:
                     int(elem)
-                except ValueError:
+                except (ValueError, TypeError):
                     if elem.__class__ in missing_cls or elem in self.missing_str:
                         self.has_missing = True
                     else:
                         return False
         return res
-#registerTypeScanner(StringIntScanner)
+registerTypeScanner(StringIntScanner)
 
 class SliceScanner(TypeScanner):
     __doc__ = 'Slice scanner'
